//*************************************************************************************
//
//  SoR Scope - software to provide Oscilloscope-like with cheap Microcontrollers
//  and free PC software
//
//  Copyright (C) 2009,	Cosmin Prund
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  ----------------------------------------------------------------------------------
//
//  Information on what's in this header file:
//
//    This header files, in conjunction with the coresponding "C" file sets up the ADC
//    for the SoR Scope. In the preparation of this header file I looked arround at
//    lots of AVR include files in an attempt to reduce the number of required
//    manual configurations. I was plesently surprised to find ount that the configuration
//    registers have the same names and work in the very same way on most of AVR's!
//    This is a grate find because it allows most of the stuff to be auto-configured,
//    as opposed to manually configured.
//
//    Here's the structure of the configuration registers as this application expects it.
//    If your AVR uses this same structure for the registers then you're in luck, this
//    code will almost automagically compile!
//
//    ADMUX:	This register contains the REFS0:1 bits for selecting reference voltate.
//				REF1 might not exist on all of them Atmels, but REFS0 is there.
//
//				ADLAR - the "left adjust result" bit;
//
//				MUX0:4	-	Multiplexor configuration bits, the least significant 5 bits.
//				They're not allways all defined, but the ones that are not defined are
//				considered "researved" so one can safely write an 0 in that position.
//
//    ADCSRA:	Grate register, it includes all of the following configuration bits. Position is
//				Always the same, also that doesn't matter because I'm using the configured defines
//				for the bits so they may float arround, as long as they stay in the same register:
//
//				ADPS0:2, ADIE, ADIF, ADATE/ADFR, ADSC, ADEN
//
//    ADCSRB:	This one gets the rest of the configuration bits. Again, bit position is actually the
//				same for all devices, but bits might as well float arround since I'm using bit names
//				to configure the registers:
//
//				ADTS, ACME, MUX5
//
//    ADCL, ADCH: Those two registers hold the result of the ADC conversion.
//
//    DIDR, DIDR0, DIDR1, DIDR2: Here's where we get into a bit of trouble. The "digital input register"
//              where one may disable digital i/o for the ADC pins are not the same over the list of
//				Atmega, so unfortunatelly one needs to configure this! The configuration is done in
//              an easy way that has the extra bnefit of throwing a compile-time error on obvious erros
//              (for example: if you try to configure an ADC pin that doesn't exist on your version of
//              Atmel)
//
//    "MUX" bits: This is an other possible cause of greef. The MUX bits are all nicely done and in order
//              for the Atmega's I've read the Datasheet for, but there's no guarantee this will be the
//              same for all of them! To make sure it works for all of them cases I decided to make this
//              configurable: The line where one configures the actual ADC pin also contains the MUX mask.				
//
//*************************************************************************************

#ifndef __SCOPE_ADC__
#define __SCOPE_ADC__

#include "prepconst.h"
#include "customize.h"

// Defines the maximum numbers of ADC ports I'm setting things up for
#ifdef ADC0D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 1
#endif

#ifdef ADC1D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 2
#endif

#ifdef ADC2D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 3
#endif

#ifdef ADC3D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 4
#endif

#ifdef ADC4D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 5
#endif

#ifdef ADC5D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 6
#endif

#ifdef ADC6D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 7
#endif

#ifdef ADC7D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 8
#endif

#ifdef ADC8D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 9
#endif

#ifdef ADC9D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 10
#endif

#ifdef ADC10D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 11
#endif

#ifdef ADC11D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 12
#endif

#ifdef ADC12D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 13
#endif

#ifdef ADC13D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 14
#endif

#ifdef ADC14D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 15
#endif

#ifdef ADC15D
#undef MAX_NUM_ADC
#define MAX_NUM_ADC 160
#endif

// Counts all of the available ADC ports. "Available" means available in the hardware and configured for use
// in the "customize.h" thing
extern unsigned char AvailableAdcCount;

// Gives the configured names for the ADC's
extern char * AdcNames[MAX_NUM_ADC];

// Gives the bit mask for the direction register
extern char AdcBitMask[MAX_NUM_ADC];

// Gives the pointer to the digital direction register
extern volatile uint8_t * AdcDDRRegister[MAX_NUM_ADC];

// Gives the MUX MASK for the analog pin
extern unsigned char AdcMuxMask[MAX_NUM_ADC];

// Function to prepare the ADC
void PrepADC();

// Do some test and throw some meaningfull errors if certain registers are not found
#ifndef ADMUX
#error "The target MCU doesn't have an ADMUX register so it can't be used with this version of SoR Scope"
#endif

#ifndef ADCSRA
#error "The target MCU doesn't have an ADCSRA register so it can't be used with this version of SoR Scope"
#endif

#ifndef ADCSRB
#error "The target MCU doesn't have an ADCSRB register so it can't be used with this version of SoR Scope"
#endif

// Some MACRO definitions to make things smoking fast
#define ENABLE_ADC {ADCSRA |= (1 << ADEN);}
#define DISABLE_ADC {ADCSRA &= ~(1<<ADEN);}
#define START_ADC {ADCSRA |= (1<<ADSC);}
#define IS_ADC_CONVERSION_GOING bit_is_set(ADCSRA, ADSC)

void AdcLeftAdjustResult();
void AdcRightAdjustResult();

#endif
